#include "neocd.h"
#include "Cyclone.h"

struct Cyclone cyclone;

static unsigned int MyCheckPc(unsigned int pc)
{
	pc = (pc - cyclone.membase) & 0xffffff;
	if(pc<0x200000) {
		cyclone.membase = (int)memory_region_cpu1;
	} else {
		switch(pc>>16)
		{
			/* BIOS ROM */
			case    0xc0:
			case    0xc1:
			case    0xc2:
			case    0xc3:
			case    0xc4:
			case    0xc5:
			case    0xc6:
			case    0xc7:
				cyclone.membase = (int)memory_region_user1 - 0xc00000;
				break;

			default:
				printf("error!! PC=%x\n", pc);
				break;
		}
	}
  	return cyclone.membase+pc; /* New program counter */
}

void m68k_pulse_reset(void)
{
	static int init = 1;
	if(init) {
		init = 0;
		CycloneInit();
	}

	memset(&cyclone, 0,sizeof(cyclone));
	cyclone.checkpc=MyCheckPc;
	cyclone.read8  =m68000_read_memory_8;
	cyclone.read16 =m68000_read_memory_16;
	cyclone.read32 =m68000_read_memory_32;
	cyclone.write8 =m68000_write_memory_8;
	cyclone.write16=m68000_write_memory_16;
	cyclone.write32=m68000_write_memory_32;
	cyclone.fetch8  =m68000_read_memory_8;
	cyclone.fetch16 =m68000_read_memory_16;
	cyclone.fetch32 =m68000_read_memory_32;
  	cyclone.srh=0x27; /* Set supervisor mode */
	cyclone.irq=0x00;
  	cyclone.a[7]=cyclone.read32(0); /* Get Stack Pointer */
  	cyclone.membase=0;
  	cyclone.pc=cyclone.checkpc(cyclone.read32(4)); /* Get Program Counter */
}

unsigned int m68000_get_reg(void* context, m68k_register_t regnum)
{
	switch(regnum)
	{
		case M68K_D0:	return cyclone.d[0];
		case M68K_D1:	return cyclone.d[1];
		case M68K_D2:	return cyclone.d[2];
		case M68K_D3:	return cyclone.d[3];
		case M68K_D4:	return cyclone.d[4];
		case M68K_D5:	return cyclone.d[5];
		case M68K_D6:	return cyclone.d[6];
		case M68K_D7:	return cyclone.d[7];
		case M68K_A0:	return cyclone.a[0];
		case M68K_A1:	return cyclone.a[1];
		case M68K_A2:	return cyclone.a[2];
		case M68K_A3:	return cyclone.a[3];
		case M68K_A4:	return cyclone.a[4];
		case M68K_A5:	return cyclone.a[5];
		case M68K_A6:	return cyclone.a[6];
		case M68K_A7:	return cyclone.a[7];
		case M68K_PC:	return cyclone.pc - cyclone.membase;
		default:			printf("m68000_get_reg Error!! %d.\n", regnum); return 0;
	}
	return 0;
}

void m68000_set_reg(m68k_register_t regnum, unsigned int value)
{
	switch(regnum)
	{
		case M68K_A7:	cyclone.a[7] = value; return;
		case M68K_PC:	cyclone.pc = cyclone.checkpc(value + cyclone.membase); return;
		case M68K_SR:	cyclone.srh = value >> 8; return;
		case M68K_USP:
		case M68K_ISP:	cyclone.osp = value; return;
		default:			printf("m68000_set_reg Error!! %d, %x.\n", regnum, value); return;
	}
}


void m68k_set_irq(unsigned int level)
{
	if(level > cyclone.irq)
		cyclone.irq = level;
/*
	if (level >= 1 && level <= 7)
		pending_interrupts |= 1 << (24 + level);
*/
}

/* NeoCD declarations */
extern int img_display;

int m68k_execute(int cycles)
{
	//MC68000_ICount = cycles;
/*
	if (pending_interrupts & 0xff000000) {
		while ((MC68000_ICount > 0) && (pending_interrupts & 0xff000000))
		{
			int level, mask = 0x80000000;
			for (level = 7; level; level--, mask >>= 1)
				if (pending_interrupts & mask)
					break;
   			if(level>cyclone.irq)
   			{
   				pending_interrupts &= ~(1 << (level + 24));
   				cyclone.irq = level;
   			}
			cyclone.cycles=46;
			printf("Int Run PC=%x, IRQ=%d.\n", cyclone.pc - cyclone.membase, cyclone.irq);
			CycloneRun(&cyclone);
			MC68000_ICount-=(46-cyclone.cycles);
		}
	}

	if (MC68000_ICount > 0) {
		printf("Run PC=%x.\n", cyclone.pc - cyclone.membase);
		cyclone.cycles=MC68000_ICount;
		CycloneRun(&cyclone);
		MC68000_ICount = cyclone.cycles;
	}
*/

	cyclone.cycles=cycles;
	while(cyclone.cycles > 0) {
		CycloneRun(&cyclone);
		//printf("Last OpCode %4x, cycles=%d.\n", cyclone.lastest_opcode, cyclone.cycles);
		switch (cyclone.lastest_opcode & 0xffff) {
			case 0xfac0: cdrom_load_files(); break;
			case 0xfac1: neogeo_cdda_control(); break;
			case 0xfac2: neogeo_exit(); break;
		}
	}

	//printf("PC=%x.(%d, %d)\n", cyclone.pc - cyclone.membase, cycles, cycles - cyclone.cycles);
   	return (cycles - cyclone.cycles);
}

void m68000_execute2(unsigned int start, unsigned int break_point)
{
	int nest_counter = 0;
	unsigned int pc, old_pc, opcode;
	struct Cyclone cyc_backup = cyclone;

	cyclone.pc = cyclone.checkpc(start + cyclone.membase);
	cyclone.a[5] = 0x108000;
	cyclone.a[7] -= 4 * 8 * 2;

	while ((pc = m68000_get_reg(NULL, M68K_PC)) != break_point)
	{
		opcode = m68000_read_memory_16(pc);
		if (opcode == 0x4e75)
		{
			// rts
			nest_counter--;
			if (nest_counter < 0) break;
		}
		else if (opcode == 0x6100)
		{
			// bsr 16
			nest_counter++;
		}
		else if ((opcode & 0xff00) == 0x6100)
		{
			// bsr 8
			nest_counter++;
		}
		else if ((opcode & 0xffc0) == 0x4e80)
		{
			// jsr
			nest_counter++;
		}

		cyclone.cycles = 1;
		CycloneRun(&cyclone);
	}

	cyclone = cyc_backup;
}
