#ifndef M68KINTRF_H
#define M68KINTRF_H

#define M68K_IRQ_NONE 0
#define M68K_IRQ_1    1
#define M68K_IRQ_2    2
#define M68K_IRQ_3    3
#define M68K_IRQ_4    4
#define M68K_IRQ_5    5
#define M68K_IRQ_6    6
#define M68K_IRQ_7    7

/* Registers used by m68000_get_reg() and m68000_set_reg() */
typedef enum
{
	/* Real registers */
	M68K_D0,		/* Data registers */
	M68K_D1,
	M68K_D2,
	M68K_D3,
	M68K_D4,
	M68K_D5,
	M68K_D6,
	M68K_D7,
	M68K_A0,		/* Address registers */
	M68K_A1,
	M68K_A2,
	M68K_A3,
	M68K_A4,
	M68K_A5,
	M68K_A6,
	M68K_A7,
	M68K_PC,		/* Program Counter */
	M68K_SR,		/* Status Register */
	M68K_SP,		/* The current Stack Pointer (located in A7) */
	M68K_USP,		/* User Stack Pointer */
	M68K_ISP,		/* Interrupt Stack Pointer */
	M68K_MSP,		/* Master Stack Pointer */
	M68K_SFC,		/* Source Function Code */
	M68K_DFC,		/* Destination Function Code */
	M68K_VBR,		/* Vector Base Register */
	M68K_CACR,		/* Cache Control Register */
	M68K_CAAR,		/* Cache Address Register */

	/* Assumed registers */
	/* These are cheat registers which emulate the 1-longword prefetch
	 * present in the 68000 and 68010.
	 */
	M68K_PREF_ADDR,	/* Last prefetch address */
	M68K_PREF_DATA,	/* Last prefetch data */

	/* Convenience registers */
	M68K_PPC,		/* Previous value in the program counter */
	M68K_IR,		/* Instruction register */
	M68K_CPU_TYPE	/* Type of CPU being run */
} m68k_register_t;

#if 0
/* Read from anywhere */
u8  m68000_read_memory_8(u32 address);
u16 m68000_read_memory_16(u32 address);
u32 m68000_read_memory_32(u32 address);

/* Write to anywhere */
void m68000_write_memory_8(u32 address, u8 value);
void m68000_write_memory_16(u32 address, u16 value);
void m68000_write_memory_32(u32 address, u32 value);
#endif

/* Do whatever initialisations the core requires.  Should be called
 * at least once at init time.
 */
void m68k_init(void);

/* Pulse the RESET pin on the CPU.
 * You *MUST* reset the CPU at least once to initialize the emulation
 */
void m68k_pulse_reset(void);

/* execute num_cycles worth of instructions.  returns number of cycles used */
int m68k_execute(int num_cycles);

/* Set the IPL0-IPL2 pins on the CPU (IRQ).
 * A transition from < 7 to 7 will cause a non-maskable interrupt (NMI).
 * Setting IRQ to 0 will clear an interrupt request.
 */
void m68k_set_irq(u32 int_level);


/* Peek at the internals of a CPU context.  This can either be a context
 * retrieved using m68k_get_context() or the currently running context.
 * If context is NULL, the currently running CPU context will be used.
 */
u32 m68000_get_reg(void* context, m68k_register_t reg);

/* Poke values into the internals of the currently running CPU context */
void m68000_set_reg(m68k_register_t reg, u32 value);

#endif /* M68KINTRF_H */
