/**
 * NeoCD/SDL main header file
 **
 * 2003 Fosters
 */
 
#ifndef NEOCD_H
#define NEOCD_H

#define TICKSPERSEC 1000000UL
#define FPS 60
#define REFRESHTIME (TICKSPERSEC / FPS)
typedef unsigned long long TICKER;

#ifdef WIN32
#define INLINE __inline
#endif

#define SAMPLE_RATE    22050

#include <SDL.h>

#define s8  signed char
#define s16 signed short
#define s32 signed long

#define u8  unsigned char
#define u16 unsigned short
#define u32 unsigned long

#define ALIGN_DATA		__attribute__((aligned(4)))
#define MEM_ALIGN		4

#ifdef WIN32
#define memalign(x, l) malloc(l)
#endif

#define Limit(val, max, min)                    \
{                                               \
        if (val > max) val = max;               \
        else if (val < min) val = min;          \
}

#ifndef MAX_PATH
#define MAX_PATH 4096
#endif

#define neogeo_boot_bios 0

#include "driver/memory.h"
#include "driver/memintrf.h"
#include "driver/driver.h"
#include "driver/inptport.h"

#ifdef MC68000
#include "m68k/mc68000/m68k.h"
#elif defined(CYCLONE)
#include "m68k/cyclone/m68k.h"
#endif

#ifdef CZ80
#include "z80/cz80/z80intrf.h"
#elif defined(DRZ80)
#include "z80/drz80/z80intrf.h"
#elif defined(MZ80)
#include "z80/mz80/z80intrf.h"
#endif

#include "video/video.h"
#include "sound/sound.h"
#include "cdrom/cdrom.h"

/*-- Version, date & time to display on startup ----------------------------*/
#define VERSION1 "NeoCD/2X"
#define VERSION2 "Compiled on: "__DATE__" "__TIME__

/*-- functions -------------------------------------------------------------*/

/*-- globals ---------------------------------------------------------------*/
extern char skip_next_frame;
extern int img_display;
extern int running;

extern char game_name[16];
extern char game_dir[MAX_PATH];
extern char launchDir[MAX_PATH];

void now_loading();
void frameskip_reset();
u32 reset_joystick(int num);
u32 read_joystick(int num);

#endif /* NEOCD_H */
