/***************************************************************************

  2610intf.c

  The YM2610 emulator supports up to 2 chips.
  Each chip has the following connections:
  - Status Read / Control Write A
  - Port Read / Data Write A
  - Control Write B
  - Data Write B

***************************************************************************/

#include <stdio.h>
#include "2610intf.h"
#include "memory.h"
#include "timer.h"

/*------------------------- YM2610 -------------------------------*/
//static void TimerHandler(int c, int count, double stepTime)
static void TimerHandler(int c, int count, u32 stepTime)
{
	if (count == 0) {		/* Reset FM Timer */
		set_timer(c, 0);
	} else {			/* Start FM Timer */
		set_timer(c, count * stepTime);
	}
}

void FMTimerInit(void)
{
    free_all_timer();
}

int YM2610_sh_start(void)
{
    int rate = SAMPLE_RATE;

	FMTimerInit();

	/**** initialize YM2610 ****/
    YM2610Init(8000000, rate, memory_region_sound1, 0x100000, TimerHandler, neogeo_sound_irq);
    return 0;
}


/************************************************/
/* Sound Hardware Stop				*/
/************************************************/
void YM2610_sh_stop(void)
{
}

/* reset */
void YM2610_sh_reset(void)
{
    YM2610Reset();
}

/************************************************/
/* Status Read for YM2610 - Chip 0		*/
/************************************************/
Uint32 YM2610_status_port_A_r(Uint32 offset)
{
    return YM2610Read(0);
}

Uint32 YM2610_status_port_B_r(Uint32 offset)
{
    return YM2610Read(2);
}

/************************************************/
/* Port Read for YM2610 - Chip 0		*/
/************************************************/
Uint32 YM2610_read_port_r(Uint32 offset)
{
    return YM2610Read(1);
}


/************************************************/
/* Control Write for YM2610 - Chip 0		*/
/* Consists of 2 addresses			*/
/************************************************/
void YM2610_control_port_A_w(Uint32 offset, Uint32 data)
{
    YM2610Write(0, data);
}

void YM2610_control_port_B_w(Uint32 offset, Uint32 data)
{
    YM2610Write(2, data);
}

/************************************************/
/* Data Write for YM2610 - Chip 0		*/
/* Consists of 2 addresses			*/
/************************************************/
void YM2610_data_port_A_w(Uint32 offset, Uint32 data)
{
    YM2610Write(1, data);
}

void YM2610_data_port_B_w(Uint32 offset, Uint32 data)
{
    YM2610Write(3, data);
}

