/**************************************
****   CDAUDIO.C  -  CD-DA Player  ****
**************************************/

//-- Include files -----------------------------------------------------------
//#include <SDL.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "neocd.h"
#include "cdaudio.h"
#include "filer.h"

#ifndef MAX_PATH
#define MAX_PATH 1024
#endif

//extern int neogeo_mp3_enable;
extern int mp3_play(char *name);
extern void mp3_stop();
extern char LastPath[MAX_PATH];

#define debug_log(str) printf(str);

//-- Public Variables --------------------------------------------------------
int cdda_first_drive=0;
int cdda_current_drive=0;
int cdda_current_track=0;
int cdda_current_frame=0;
int cdda_volume=0;
int cdda_disabled=0;
volatile int cdda_playing=0;
volatile int cdda_autoloop=0;
static int cdda_pathset;
static char cdda_path[MAX_PATH];

//----------------------------------------------------------------------------
int	cdda_init(void)
{
	cdda_current_track = 0;
	cdda_playing = 0;
	cdda_disabled = 0;
	cdda_pathset = 0;

	mp3_init();

	return 1;
}

//----------------------------------------------------------------------------
int cdda_play(int track)
{
	char path[MAX_PATH];
	char name[16];
	char ret[MAX_PATH];

	if(cdda_disabled) { printf("cdda disabled.\n");return 0; }

	printf("pl.trk.%d\n",track);

	if(cdda_playing && cdda_current_track == track) {
		if (cdda_playing == 2)
			cdda_playing = 1;
		return 1;
	}

	mp3_stop();
	cdda_current_track = track;
	sprintf(name, "%02d.mp3", track);
	cdda_disabled = 1;
	
	while(!cdda_pathset) {
		strcpy(path, LastPath);
		if(find_file(name, path, ret)) {
			strcpy(cdda_path, path);
			cdda_pathset = 1;
			break;
		}
		
		if(archive_mode != MODE_RAW) {
			char *p;
			strcpy(path, game_dir);
			p = strrchr(path, '.');
			if(p == NULL) {
				strcat(path, "/");
			} else {
				p[0] = '/';
				p[1] = '\0';
			}
			if(find_file(name, path, ret)) {
				strcpy(cdda_path, path);
				cdda_pathset = 1;
				break;
			}
		}

		sprintf(path, "%sneocd.mp3/", LastPath);
		if(find_file(name, path, ret)) {
			strcpy(cdda_path, path);
			cdda_pathset = 1;
			break;
		}

		sprintf(path, "%smp3/", LastPath);
		if(find_file(name, path, ret)) {
			strcpy(cdda_path, path);
			cdda_pathset = 1;
			break;
		}

		sprintf(path, "%sbgm/", LastPath);
		if(find_file(name, path, ret)) {
			strcpy(cdda_path, path);
			cdda_pathset = 1;
			break;
		}
		
		cdda_pathset = 0;
		cdda_disabled = 1;
		break;
	}
	
	if(cdda_pathset) {
		strcpy(path, cdda_path);
		if (find_file(name, path, ret)) {
			debug_log("cdda play - mp3 found\n");
			strcat(path, ret);
			cdda_disabled = mp3_play(path);
		} else {
			cdda_disabled = 1;
		}
	}
	
	if(cdda_disabled) {
		debug_log("cdda play - mp3 not found\n");
		return 0;
	}

	cdda_playing = 1;
	return 1;
}

//----------------------------------------------------------------------------
void cdda_pause(void)
{
	if(cdda_disabled) return;
	cdda_playing = 2;
}

//----------------------------------------------------------------------------
void cdda_resume(void)
{
	if(cdda_disabled) return;
	cdda_playing = 1;
}

//----------------------------------------------------------------------------
void cdda_stop(void)
{
	if(cdda_disabled) return;
	mp3_stop();
	cdda_playing = 0;
}

//----------------------------------------------------------------------------
void cdda_shutdown(void)
{
	if(cdda_disabled) return;
	mp3_shutdown();
}

//----------------------------------------------------------------------------
void neogeo_do_cdda( int command, int track_number_bcd)
{
	int track_number;
	int offset;

	if ((command == 0)&&(track_number_bcd == 0))
		return;

	switch( command )
	{
		case 0:
		case 1:
		case 4:
		case 5:
			track_number = ((track_number_bcd>>4)*10) + (track_number_bcd&0x0F);
            if ((track_number>1) && (track_number<=99)) {
				cdda_autoloop = !(command&1);
				cdda_play(track_number);
			}
			break;
		case 2:
		case 6:
			cdda_pause();
			break;
		case 3:
		case 7:
			cdda_resume();
			break;
	}
}

//----------------------------------------------------------------------------
void neogeo_cdda_check(void)
{
	unsigned int offset = m68000_read_memory_32(0x108000 + 0x76ea);

	if (offset)
	{
		unsigned char command, track;

		offset  = (offset - 0xe00000) >> 1;

		command = memory_region_cpu2[offset + 0];
		track   = memory_region_cpu2[offset + 1];

		memory_region_cpu2[offset + 0] = 0;
		memory_region_cpu2[offset + 1] = 0;

		if ((command <= 7) && (command || track))
			neogeo_do_cdda(command, track);
	}
}

//----------------------------------------------------------------------------
void neogeo_cdda_control(void)
{
	int command, track;
	u32 offset;

	command = m68000_get_reg(NULL, M68K_D0);
	track   = command & 0xff;
	command = (command >> 8) & 0xff;

	if ((command & 2) == 0)
	{
		m68000_write_memory_8(0x108000 + 0x764b, track);
		m68000_write_memory_8(0x108000 + 0x76f8, track);
		m68000_write_memory_8(0x108000 + 0x76f7, command);
	}
	m68000_write_memory_8(0x108000 + 0x76f6, command);

	if ((command <= 7) && (command || track))
		neogeo_do_cdda(command, track);

	offset = m68000_read_memory_32(0x108000 + 0x76ea);

	if (offset)
	{
		m68000_write_memory_8(0x108000 + 0x7678, 0x01);

		if (command <= 7)
		{
			offset  = (offset - 0xe00000) >> 1;
			memory_region_cpu2[offset + 0] = 0;
			memory_region_cpu2[offset + 1] = 0;
		}
	}
}
