/* 
 *  MadWrapper - SMPEG compatible wrapper functions for MAD: Mpeg Audio Decoder
 *
 *  Copyright (c) 2001-2003 Ogapee. All rights reserved.
 *
 *  ogapee@aqua.dti2.ne.jp
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <SDL.h>
#include <mad.h>
#include "neocd.h"
//#include "cdaudio/cdaudio.h"
//#include "sound/sound.h"

#define EnterSound() SDL_LockMutex(mad_mutex)
#define LeaveSound() SDL_UnlockMutex(mad_mutex)

#define DEFAULT_AUDIOBUF  4096
#define INPUT_BUFFER_SIZE (5*8192)

typedef struct _mp3_mad{
	SDL_RWops *src;
	Uint32 length;
	struct mad_stream Stream;
	struct mad_frame  Frame;
	struct mad_synth  Synth;
	int volume;

	unsigned char *input_buf;
	unsigned char *left_buf;
	unsigned char *right_buf;
	unsigned long output_buf_index;
} mp3_mad;

mp3_mad *mad = NULL;
SDL_RWops *mad_src = NULL;
int MP3_playing = 0;
SDL_mutex *mad_mutex = NULL;

extern volatile int cdda_playing;
extern volatile int cdda_autoloop;

void mp3_stop();

mp3_mad* mp3_mad_new( const char *file, void* info, int sdl_audio );
mp3_mad* mp3_mad_new_rwops( SDL_RWops *src, void* info, int sdl_audio );
int mp3_mad_fill( void *userdata, int len );
void mp3_mad_delete( mp3_mad *mad );
void mp3_mad_setvolume( mp3_mad *mad, int volume );

void mp3_update(Sint16 *left, Sint16 *right, int len) {
	len <<= 1;
	memset(left, 0, len);
	memset(right, 0, len);

	EnterSound();
	if(MP3_playing && (cdda_playing == 1)) {
		int mp3len = mp3_mad_fill( mad, len );

		if(mp3len > len) {
			memcpy(left, mad->left_buf, len);
			memcpy(right, mad->right_buf, len);

			memcpy(mad->left_buf, mad->left_buf + len, mp3len - len);
			memcpy(mad->right_buf, mad->right_buf + len, mp3len - len);

			mad->output_buf_index = mp3len - len;
		} else {
			memcpy(left, mad->left_buf, mp3len);
			memcpy(right, mad->right_buf, mp3len);

			mad->output_buf_index = 0;
		}
	}
	LeaveSound();
}

int mp3_play(char *name) {
	mp3_stop();
	printf("MP3 File: %s\n", name);

	if(mad) mp3_mad_delete(mad);

	mad_src = SDL_RWFromFile(name, "rb");
	if(mad_src) {
		mad = mp3_mad_new_rwops(mad_src, 0, 0);
		MP3_playing = 1;
		return 0;
	}

	mad = NULL;
	return 1;
}

void mp3_stop() {
	EnterSound();
	MP3_playing = 0;
	LeaveSound();
}

void mp3_init() {
	MP3_playing = 0;
	if(mad) mp3_mad_delete(mad);
	mad_src = NULL;
	mad = NULL;
	mad_mutex = SDL_CreateMutex();
	//stream_init_multi(2, 0, mp3_update);
}

void mp3_shutdown() {
	mp3_stop();
	if(mad) mp3_mad_delete(mad);
	mad_src = NULL;
	mad = NULL;
	SDL_DestroyMutex(mad_mutex);
	mad_mutex = NULL;
}

/*
void SetVolumeMP3() {
	if(mad_mutex && mad) {
		EnterSound();
		if(main_volume < 0) main_volume = 0;
		if(main_volume > 100) main_volume = 100;
		mp3_mad_setvolume(mad, main_volume);
		LeaveSound();
	}
}
*/

static inline unsigned short MadFixedToUshort( mad_fixed_t Fixed )
{
	Fixed = Fixed >> (MAD_F_FRACBITS-15);
	return ((unsigned short)Fixed);
}

static mp3_mad* init( SDL_RWops *src )
{
	mp3_mad *mad = malloc(sizeof(mp3_mad));

	SDL_RWseek( src, 0, SEEK_END );
	mad->length = SDL_RWtell( src );
	SDL_RWseek( src, 0, SEEK_SET );

	mad->src = src;

	mad_stream_init( &mad->Stream );
	mad_frame_init( &mad->Frame );
	mad_synth_init( &mad->Synth );
	mad->volume = 64;

	mad->input_buf = (unsigned char*)malloc(INPUT_BUFFER_SIZE);
	mad->left_buf = (unsigned char*)malloc(1152*4*5);
	mad->right_buf = (unsigned char*)malloc(1152*4*5);
	mad->output_buf_index = 0;

	return mad;
}

mp3_mad* mp3_mad_new( const char *file, void* info, int sdl_audio )
{
	SDL_RWops *src;

	src = SDL_RWFromFile( file, "rb" );
	if ( !src ) return NULL;

	return init( src );
}

mp3_mad* mp3_mad_new_rwops( SDL_RWops *src, void* info, int sdl_audio )
{
	return init( src );
}

int mp3_mad_fill( void *userdata, int len )
{
	mp3_mad *mad = (mp3_mad*)userdata;

	size_t         ReadSize = 1, Remaining;
	unsigned char  *ReadStart;

	while( (cdda_playing == 1) && (mad->output_buf_index < len) ) {
		if( mad->Stream.buffer==NULL || mad->Stream.error==MAD_ERROR_BUFLEN ){
	
			if ( mad->Stream.next_frame != NULL ){
				Remaining = mad->Stream.bufend - mad->Stream.next_frame;
				memmove( mad->input_buf, mad->Stream.next_frame, Remaining);
				ReadStart = mad->input_buf + Remaining;
				ReadSize  = INPUT_BUFFER_SIZE - Remaining;
			} else{
				ReadSize  = INPUT_BUFFER_SIZE;
				ReadStart = mad->input_buf;
				Remaining = 0;
			}

			ReadSize = SDL_RWread( mad->src, ReadStart, 1, ReadSize );
			if ( ReadSize <= 0 ) { // end of stream
				if(cdda_autoloop) {
					SDL_RWseek( mad->src, 0, SEEK_SET );
					continue;
				}
				break;
			}

			mad_stream_buffer( &mad->Stream, mad->input_buf, ReadSize + Remaining );
			mad->Stream.error = MAD_ERROR_NONE;
		}

		if ( mad_frame_decode( &mad->Frame,&mad->Stream ) ){
			if ( MAD_RECOVERABLE( mad->Stream.error ) ||
				mad->Stream.error == MAD_ERROR_BUFLEN ){
				continue;
			}
			else{
				printf( "unrecoverable frame level error (%s).\n", mad_stream_errorstr(&mad->Stream) );
				return 0; // error
			}
		}

		if (( mad->Frame.header.samplerate == 44100 ) && (SAMPLE_RATE == 22050))
			mad->Frame.options |= MAD_OPTION_HALFSAMPLERATE;

		mad_synth_frame( &mad->Synth, &mad->Frame );
	
		char *left_ptr = (char*)mad->left_buf + mad->output_buf_index;
		char *right_ptr = (char*)mad->right_buf + mad->output_buf_index;
		int i;

		for ( i=0 ; i<mad->Synth.pcm.length ; i++ ){
			unsigned short  Sample;

			/* Left channel */
			Sample=MadFixedToUshort( mad->Synth.pcm.samples[0][i] );
#if SDL_BYTEORDER == SDL_LIL_ENDIAN
			*(left_ptr++) = Sample & 0xff;
			*(left_ptr++) = Sample >> 8;
#else
			*(left_ptr++) = Sample >> 8;
			*(left_ptr++) = Sample & 0xff;
#endif

			/* Right channel, if exist. */
			if ( MAD_NCHANNELS(&mad->Frame.header)==2 )
				Sample=MadFixedToUshort( mad->Synth.pcm.samples[1][i] );
#if SDL_BYTEORDER == SDL_LIL_ENDIAN
			*(right_ptr++) = Sample & 0xff;
			*(right_ptr++) = Sample >> 8;
#else
			*(right_ptr++) = Sample >> 8;
			*(right_ptr++) = Sample&0xff;
#endif
		}
		mad->output_buf_index += mad->Synth.pcm.length * 2;
	}

	return mad->output_buf_index;
}

void mp3_mad_delete( mp3_mad *mad )
{
	mad_synth_finish( &mad->Synth );
	mad_frame_finish( &mad->Frame );
	mad_stream_finish( &mad->Stream );
	
	free(mad->input_buf);
	free(mad->left_buf);
	free(mad->right_buf);
	SDL_RWclose( mad->src );
	free(mad);
}

void mp3_mad_setvolume( mp3_mad *mad, int volume )
{
	if ( volume < 0) volume = 0;
	if ( volume > 100) volume = 100;
	if ( (volume >= 0) && (volume <= 100) ) {
		mad->volume = (volume*SDL_MIX_MAXVOLUME) / 100;
	}
}
