/*  gngeo a neogeo emulator
 *  Copyright (C) 2001 Peponas Mathieu
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <string.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef GP2X
#include <fcntl.h>
#include <pthread.h>
#include <sys/select.h>
#include <linux/soundcard.h>
#endif
#include "neocd.h"
//#include "streams.h"
#include "sound.h"

#define BUFFER_LEN 16384
#define NB_SAMPLES 1024

int sound_volume;
int sound = 0;
Sint16 play_buffer[BUFFER_LEN];
extern int filer_option;

#ifdef GP2X
static int sound_fd = -1;
static int frag_size = 0;
static int sound_play = 0;
static int sound_id = -1;
static pthread_t sound_th;

void *sound_thread(void *data)
{
	fd_set fdset;
	struct timeval timeout;

	printf("sound thread start...\n");
    while(sound_play)
    {
		if (filer_option != 1) {
            usleep(1000000);
			continue;
		}

    	FD_ZERO(&fdset);
    	FD_SET(sound_fd, &fdset);
    	timeout.tv_sec = 10;
    	timeout.tv_usec = 0;

		if ( select(sound_fd+1, NULL, &fdset, NULL, &timeout) <= 0 ) {
            sound_play = 0;
        } else if(sound_play && sound) {
			sound_update(frag_size >> 2);
			write(sound_fd, play_buffer, frag_size);
        } else {
            usleep(10000);
        }
    }
	printf("sound thread terminated...\n");

    pthread_exit(0);
}
#else
SDL_AudioSpec desired;

void update_sdl_stream(void *userdata, Uint8 * stream, int len)
{
	SDL_LockAudio();
    sound_update(len >> 2);
    memcpy(stream, (Uint8 *) play_buffer, len);
	SDL_UnlockAudio();
}
#endif

int sound_init(void)
{
#ifdef GP2X
	sound_play = sound = 0;
	if (sound_fd >= 0) close(sound_fd);
	if (sound_id >= 0) pthread_join(sound_th, NULL);

	sound_fd = open("/dev/dsp", O_WRONLY|O_ASYNC);
	if (sound_fd == -1) {
		fprintf(stderr, "Couldn't open /dev/dsp device.\n");
		return 0;
    } else {
    	int rate, bits, stereo, frag;
    	rate = SAMPLE_RATE;
    	bits = 16;
    	stereo = 1;

    	ioctl(sound_fd, SNDCTL_DSP_SPEED,  &rate);
    	ioctl(sound_fd, SNDCTL_DSP_SETFMT, &bits);
    	ioctl(sound_fd, SNDCTL_DSP_STEREO, &stereo);

    	frag = 11;
		if(rate > 11025) frag++;
		if(rate > 22050) frag++;
		frag_size = 1 << frag;
    	frag |= 2 << 16;
    	ioctl(sound_fd, SNDCTL_DSP_SETFRAGMENT, &frag);
    }

    sound_play = 1;
    sound_id = pthread_create(&sound_th, NULL, sound_thread, NULL);
    if (sound_id < 0) {
        fprintf(stderr, "Sound thread create failed.");
		sound_play = 0;
        return 0;
    }
#else
	if(SDL_InitSubSystem(SDL_INIT_AUDIO)) {
		fprintf(stderr,"Couldn't Start Sound.\n");
		return 0;
	}

	desired.freq = SAMPLE_RATE;
	desired.samples = NB_SAMPLES;

#ifdef WORDS_BIGENDIAN
    desired.format = AUDIO_S16MSB;
#else	/* */
    desired.format = AUDIO_S16;
#endif	/* */
	desired.channels = 2;
	desired.callback = update_sdl_stream;
	desired.userdata = NULL;
	SDL_OpenAudio(&desired, NULL);
#endif
	return 1;
}

void sound_pause(int pause) {
	if(pause == 2)
		sound = !sound;
	else
		sound = !pause;

#ifndef GP2X
	SDL_PauseAudio(!sound);
#endif
}

void sound_shutdown(void) {
#ifdef GP2X
    sound_play = sound = 0;
    //pthread_join(sound_th, NULL);
    close(sound_fd);
	sound_fd = sound_id = -1;
#else
	SDL_PauseAudio(1);
	SDL_CloseAudio();
	SDL_QuitSubSystem(SDL_INIT_AUDIO);
#endif
}

void sound_set_volume(int volume) {
#ifdef GP2X
	static int prev = -1;

	if(volume < 0) volume = 0;
	if(volume > 90) volume = 90;

	if(volume != prev) {
	    int mixer = open("/dev/mixer", O_RDWR);
        prev = sound_volume = volume;

        if(mixer >= 0) {
            volume |= volume << 8;
        	ioctl(mixer, SOUND_MIXER_WRITE_PCM, &volume);
        	close(mixer);
        }
	}
#endif
}
