// NeoGeo tile draw for ARM

#include <stdio.h>
#include <string.h>

FILE *fp;
char skip[16][16] =
{
    { 0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0 },
    { 0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0 },
    { 0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0 },
    { 0,0,1,0,1,0,0,0,1,0,0,0,1,0,0,0 },
    { 0,0,1,0,1,0,0,0,1,0,0,0,1,0,1,0 },
    { 0,0,1,0,1,0,1,0,1,0,0,0,1,0,1,0 },
    { 0,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0 },
    { 1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0 },
    { 1,0,1,0,1,0,1,0,1,1,1,0,1,0,1,0 },
    { 1,0,1,1,1,0,1,0,1,1,1,0,1,0,1,0 },
    { 1,0,1,1,1,0,1,0,1,1,1,0,1,0,1,1 },
    { 1,0,1,1,1,0,1,1,1,1,1,0,1,0,1,1 },
    { 1,0,1,1,1,0,1,1,1,1,1,0,1,1,1,1 },
    { 1,1,1,1,1,0,1,1,1,1,1,0,1,1,1,1 },
    { 1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1 },
    { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 }
};
int skip_i[16] ={
	0x0080,	0x0880,	0x0888,	0x2888,	0x288a,	0x2a8a,	0x2aaa,	0xaaaa,
	0xaaea,	0xbaea,	0xbaeb,	0xbbeb,	0xbbef,	0xfbef,	0xfbff,	0xffff
};

void draw_char()
{
	int i;

	fprintf(fp, ".macro DRAW_LINE\n");

	fprintf(fp, "\tands r1,r5,r0\n");
	fprintf(fp, "\tldrne r6,[r3, r1, lsl #2]\n");
	fprintf(fp, "\tstrneh r6,[r2]\n");
	for (i=1;i<8;i++)
	{
		fprintf(fp, "\tands r1,r5,r0, lsr #%d\n", i * 4);
		fprintf(fp, "\tldrne r6,[r3, r1, lsl #2]\n");
		fprintf(fp, "\tstrneh r6,[r2, #%d]\n", i * 2);
	}

	fprintf(fp, "\tadd r2,r2,#640\n");
	fprintf(fp, ".endm\n");

	fprintf(fp, "\t.ltorg\n");
	fprintf(fp, "\t.align 4\n");
	fprintf(fp, "draw_one_char_arm:\n");
	fprintf(fp, "\tstmdb sp!,{r4-r6}\n");

	fprintf(fp, "\tldr r3, = video_paletteram_pc\n");
	fprintf(fp, "\tldr r3,[r3]\n");
	fprintf(fp, "\tadd r3,r3,r1, lsl #2	;@ r3=pal\n");

	fprintf(fp, "\tldr r4, = neogeo_fix_memory\n");
	fprintf(fp, "\tldr r4,[r4]\n");
	fprintf(fp, "\tadd r4,r4,r0, lsl #5	;@ r4=gfx\n");

	fprintf(fp, "\tmov r5,#0xF\n");

	fprintf(fp, "\tldr r0,[r4]\n");
	fprintf(fp, "\tDRAW_LINE\n");
	for (i=1;i<8;i++)
	{
		fprintf(fp, "\tldr r0,[r4, #%d]\n", i * 4);
		fprintf(fp, "\tDRAW_LINE\n");
	}

	fprintf(fp, "\tldmia sp!,{r4-r6}\n");
	fprintf(fp, "\tmov pc,lr 		;@ return\n");
}

void draw_strneh(int x)
{
	if (x) {
		fprintf(fp, "\tstrneh r6,[r2, #%d]\n", x * 2);
	} else {
		fprintf(fp, "\tstrneh r6,[r2]\n");
	}
}

void draw_sprite()
{
	int i, x, l;

	for (i=1; i<=16; i++)
	{
		l = 0;
		fprintf(fp, ".macro DRAW_SPRITE_LINE_%d\n", i);
		if (skip_i[i-1] & 0xff00)
		{
			fprintf(fp, "\tldr r0,[r4]\n");
			if (skip[i-1][0])
			{
				fprintf(fp, "\tands r1,r5,r0\n");
				fprintf(fp, "\tldrne r6,[r3, r1, lsl #2]\n");
				draw_strneh(l++);
			}
			for (x=1;x<8;x++)
			{
				if (skip[i-1][x])
				{
					fprintf(fp, "\tands r1,r5,r0, lsr #%d\n", x * 4);
					fprintf(fp, "\tldrne r6,[r3, r1, lsl #2]\n");
					draw_strneh(l++);
				}
			}
		}

		if (skip_i[i-1] & 0xff)
		{
			fprintf(fp, "\tldr r0,[r4,#4]\n");
			if (skip[i-1][8])
			{
				fprintf(fp, "\tands r1,r5,r0\n");
				fprintf(fp, "\tldrne r6,[r3, r1, lsl #2]\n");
				draw_strneh(l++);
			}
			for (x=9;x<16;x++)
			{
				if (skip[i-1][x])
				{
					fprintf(fp, "\tands r1,r5,r0, lsr #%d\n", (x - 8) * 4);
					fprintf(fp, "\tldrne r6,[r3, r1, lsl #2]\n");
					draw_strneh(l++);
				}
			}
		}
		fprintf(fp, ".endm\n");

		l = 0;
		fprintf(fp, ".macro DRAW_SPRITE_LINE_XFLIP_%d\n", i);
		if (skip_i[i-1] & 0xff)
		{
			fprintf(fp, "\tldr r0,[r4,#4]\n");
			if (skip[i-1][15])
			{
				fprintf(fp, "\tands r1,r5,r0, lsr #28\n");
				fprintf(fp, "\tldrne r6,[r3, r1, lsl #2]\n");
				draw_strneh(l++);
			}
			for (x=14;x>8;x--)
			{
				if (skip[i-1][x])
				{
					fprintf(fp, "\tands r1,r5,r0, lsr #%d\n", (x - 8) * 4);
					fprintf(fp, "\tldrne r6,[r3, r1, lsl #2]\n");
					draw_strneh(l++);
				}
			}
			if (skip[i-1][8])
			{
				fprintf(fp, "\tands r1,r5,r0\n");
				fprintf(fp, "\tldrne r6,[r3, r1, lsl #2]\n");
				draw_strneh(l++);
			}
		}

		if (skip_i[i-1] & 0xff00)
		{
			fprintf(fp, "\tldr r0,[r4]\n");
			for (x=7;x>0;x--)
			{
				if (skip[i-1][x])
				{
					fprintf(fp, "\tands r1,r5,r0, lsr #%d\n", x * 4);
					fprintf(fp, "\tldrne r6,[r3, r1, lsl #2]\n");
					draw_strneh(l++);
				}
			}
			if (skip[i-1][0])
			{
				fprintf(fp, "\tands r1,r5,r0\n");
				fprintf(fp, "\tldrne r6,[r3, r1, lsl #2]\n");
				draw_strneh(l++);
			}
		}
		fprintf(fp, ".endm\n");

		fprintf(fp, ".macro DRAW_LINE_NORM_%d\n", i);
		fprintf(fp, "\tDRAW_SPRITE_LINE_%d\n", i);
		fprintf(fp, "\tadd r2,r2,#704\n");
		fprintf(fp, ".endm\n");
		fprintf(fp, ".macro DRAW_LINE_XFLIP_%d\n", i);
		fprintf(fp, "\tDRAW_SPRITE_LINE_XFLIP_%d\n", i);
		fprintf(fp, "\tadd r2,r2,#704\n");
		fprintf(fp, ".endm\n");
		fprintf(fp, ".macro DRAW_LINE_YFLIP_%d\n", i);
		fprintf(fp, "\tDRAW_SPRITE_LINE_%d\n", i);
		fprintf(fp, "\tsub r2,r2,#704\n");
		fprintf(fp, ".endm\n");
		fprintf(fp, ".macro DRAW_LINE_XYFLIP_%d\n", i);
		fprintf(fp, "\tDRAW_SPRITE_LINE_XFLIP_%d\n", i);
		fprintf(fp, "\tsub r2,r2,#704\n");
		fprintf(fp, ".endm\n");
	}

	fprintf(fp, ".macro DRAW_TILE name\n");
	fprintf(fp, "\tstmdb sp!,{r4-r8}\n");

	fprintf(fp, "\tmovs r8,r3               ;@ r8=zy\n");
	fprintf(fp, "\tbeq .end_\\name\n");

	fprintf(fp, "\tldr r3, = video_paletteram_pc\n");
	fprintf(fp, "\tldr r3,[r3]\n");
	fprintf(fp, "\tadd r3,r3,r1, lsl #6    ;@ r3=pal\n");

	fprintf(fp, "\tldr r4, = neogeo_spr_memory\n");
	fprintf(fp, "\tldr r4,[r4]\n");
	fprintf(fp, "\tadd r4,r4,r0, lsl #7    ;@ r4=gfx, r2=bmp\n");

	fprintf(fp, "\tldr r7, = dda_y_skip\n");
	fprintf(fp, "\tldr r7,[r7]\n");

	fprintf(fp, "\tmov r5,#0xF\n");

	fprintf(fp, ".lineloop_\\name:\n");

	fprintf(fp, "\tldrb r0,[r7], #1\n");
	fprintf(fp, "\tadd r4,r4,r0, lsl #3\n");

	fprintf(fp, "\tDRAW_LINE_\\name\n");

	fprintf(fp, "\tsubs r8, r8, #1\n");
	fprintf(fp, "\tbne .lineloop_\\name\n");

	fprintf(fp, ".end_\\name:\n");

	fprintf(fp, "\tldmia sp!,{r4-r8}\n");
	fprintf(fp, "\tmov pc,lr               ;@ return\n");
	fprintf(fp, ".endm\n");

	for (i=1; i<=16; i++)
	{
		fprintf(fp, "\t.ltorg\n");
		fprintf(fp, "\t.align 4\n");
		fprintf(fp, "draw_tile_arm_norm_%d:\n", i);
		fprintf(fp, "\tDRAW_TILE NORM_%d\n", i);

		fprintf(fp, "\t.align 4\n");
		fprintf(fp, "draw_tile_arm_xflip_%d:\n", i);
		fprintf(fp, "\tDRAW_TILE XFLIP_%d\n", i);

		fprintf(fp, "\t.align 4\n");
		fprintf(fp, "draw_tile_arm_yflip_%d:\n", i);
		fprintf(fp, "\tDRAW_TILE YFLIP_%d\n", i);

		fprintf(fp, "\t.align 4\n");
		fprintf(fp, "draw_tile_arm_xyflip_%d:\n", i);
		fprintf(fp, "\tDRAW_TILE XYFLIP_%d\n", i);
	}
}

int main()
{
	int i;

	if (fp = fopen("video_arm.s", "w"))
	{
		fprintf(fp, ".global draw_one_char_arm\n");
		for (i=1; i<=16; i++)
		{
			fprintf(fp, ".global draw_tile_arm_norm_%d\n", i);
			fprintf(fp, ".global draw_tile_arm_xflip_%d\n", i);
			fprintf(fp, ".global draw_tile_arm_yflip_%d\n", i);
			fprintf(fp, ".global draw_tile_arm_xyflip_%d\n", i);
		}
		fprintf(fp, ".extern video_paletteram_pc\n");
		fprintf(fp, ".extern neogeo_fix_memory\n");
		fprintf(fp, ".extern dda_y_skip\n");

		draw_char();
		draw_sprite();
		fclose(fp);
	}

	if (fp = fopen("video_arm.h", "w"))
	{
		for (i=1; i<=16; i++)
		{
			fprintf(fp, "extern void draw_tile_arm_norm_%d(int tile, int color, unsigned short *video, int zy);\n", i);
			fprintf(fp, "extern void draw_tile_arm_xflip_%d(int tile, int color, unsigned short *video, int zy);\n", i);
			fprintf(fp, "extern void draw_tile_arm_yflip_%d(int tile, int color, unsigned short *video, int zy);\n", i);
			fprintf(fp, "extern void draw_tile_arm_xyflip_%d(int tile, int color, unsigned short *video, int zy);\n", i);
		}
		fprintf(fp, "\n");

		fprintf(fp, "typedef void (*DrawTileFunc)(int tile, int color, unsigned short *video, int zy);\n");
		fprintf(fp, "static DrawTileFunc DrawTileARM[17][4] = {\n");
		fprintf(fp, "\t{NULL, NULL, NULL, NULL},\n");
		for (i=1; i<=16; i++)
		{
			fprintf(fp, "\t{");
			fprintf(fp, "&draw_tile_arm_norm_%d,", i);
			fprintf(fp, "&draw_tile_arm_xflip_%d,", i);
			fprintf(fp, "&draw_tile_arm_yflip_%d,", i);
			fprintf(fp, "&draw_tile_arm_xyflip_%d", i);
			fprintf(fp, "},\n");
		}
		fprintf(fp, "};\n");
		fclose(fp);
	}
/*
	if (fp = fopen("video_arm_switch.c", "w"))
	{
		fprintf(fp, "switch((zx << 2) | flip){\n");
		for (i=1; i<=16; i++)
		{
			fprintf(fp, "\tcase %d: draw_tile_arm_norm_%d(tileno, color, video_full_ptr[sy]+sx, zy);break;\n", i << 2, i);
			fprintf(fp, "\tcase %d: draw_tile_arm_xflip_%d(tileno, color, video_full_ptr[sy]+sx, zy);break;\n", (i << 2) | 1, i);
			fprintf(fp, "\tcase %d: draw_tile_arm_yflip_%d(tileno, color, video_full_ptr[sy+zy-1]+sx, zy);break;\n", (i << 2) | 2, i);
			fprintf(fp, "\tcase %d: draw_tile_arm_xyflip_%d(tileno, color, video_full_ptr[sy+zy-1]+sx, zy);break;\n", (i << 2) | 3, i);
		}
		fprintf(fp, "}\n");
		fclose(fp);
	}
*/
	return 0;
}
