/*******************************************
**** VIDEO.C - Video Hardware Emulation ****
*******************************************/

//-- Include Files -----------------------------------------------------------
#include <SDL.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "neocd.h"


//-- Defines -----------------------------------------------------------------
#define VIDEO_TEXT		0
#define VIDEO_NORMAL	1
#define	VIDEO_SCANLINES	2

//-- Imported Variables ------------------------------------------------------

//-- Global Variables --------------------------------------------------------
u16 ALIGN_DATA neogeo_vidram16[0x20000 / 2];
u16 neogeo_vidram16_offset;
u16 neogeo_vidram16_modulo;

u16 *neogeo_paletteram16;
u16 ALIGN_DATA neogeo_palettebank16[2][0x2000 / 2];
u32 neogeo_palette_index;

u32 *video_palette;
u32 ALIGN_DATA video_palettebank[2][0x2000 / 2];
u16 ALIGN_DATA video_clut16[0x8000];

u8 video_fix_usage[0x20000 / 32];
u8 video_spr_usage[0x400000 / 128];

int video_enable;
int spr_disable;
int fix_disable;

int video_mode = 0;
u16 *video_line_ptr[224];
u16 *video_full_ptr[256];

SDL_Surface		*screen;
SDL_Surface		*video_buffer;
SDL_Surface		*game_title;
Uint8			*SrcPtr;
Uint8			*DestPtr;

char *dda_x_skip;
char ddaxskip[16][16] =
{
    { 0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0 },
    { 0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0 },
    { 0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0 },
    { 0,0,1,0,1,0,0,0,1,0,0,0,1,0,0,0 },
    { 0,0,1,0,1,0,0,0,1,0,0,0,1,0,1,0 },
    { 0,0,1,0,1,0,1,0,1,0,0,0,1,0,1,0 },
    { 0,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0 },
    { 1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0 },
    { 1,0,1,0,1,0,1,0,1,1,1,0,1,0,1,0 },
    { 1,0,1,1,1,0,1,0,1,1,1,0,1,0,1,0 },
    { 1,0,1,1,1,0,1,0,1,1,1,0,1,0,1,1 },
    { 1,0,1,1,1,0,1,1,1,1,1,0,1,0,1,1 },
    { 1,0,1,1,1,0,1,1,1,1,1,0,1,1,1,1 },
    { 1,1,1,1,1,0,1,1,1,1,1,0,1,1,1,1 },
    { 1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1 },
    { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 }
};
char *dda_y_skip;
char ddayskip[256][32][16];
char ddaysize[256][32];

int snap_no;
SDL_Rect src,dest;

static u16 *sprite_scale = &neogeo_vidram16[0x10000 >> 1];
static u16 *sprite_yctrl = &neogeo_vidram16[0x10400 >> 1];
static u16 *sprite_xctrl = &neogeo_vidram16[0x10800 >> 1];

static int startline = 0;

//-- Function Prototypes -----------------------------------------------------
void video_draw_spr(int start, int end, int scanline);
void snapshot_init(void);
void video_save_snapshot(void);
void blitter(int scanline);

#ifdef ARMASM
#include "video_arm.h"
static __inline__ void draw_tile_gp2x(unsigned int tileno,int sx,int sy,int zx,int zy,int color,int flip);
#define DRAWTILE(t,x,y,rx,ry,c,xf,yf,b) draw_tile_gp2x(t,x,y,rx,ry,c,tileatr & 0x03)
#elif defined(X86ASM)
void draw_tile_i386(unsigned int tileno,int sx,int sy,int zx,int zy,
						int color,int xflip,int yflip,unsigned char *bmp);
#define DRAWTILE draw_tile_i386
#else
static __inline__ void draw_tile(unsigned int tileno,int sx,int sy,int zx,int zy,
						int color,int xflip,int yflip,unsigned char *bmp);
#define DRAWTILE draw_tile
#undef X86ASM
#endif

#define MAKECOLOR(r, g, b)	(((r & 0xf8) << 8) | ((g & 0xfc) << 3) | ((b & 0xf8) >> 3))

//----------------------------------------------------------------------------
int	neogeo_video_init(void)
{
	int y;
	unsigned short	*ptr;
	int i, r, g, b;

	for (r = 0; r < 32; r++)
	{
		for (g = 0; g < 32; g++)
		{
			for (b = 0; b < 32; b++)
			{
				int r1 = (r << 3) | (r >> 2);
				int g1 = (g << 3) | (g >> 2);
				int b1 = (b << 3) | (b >> 2);

				u16 color = ((r & 1) << 14) | ((r & 0x1e) << 7)
						  | ((g & 1) << 13) | ((g & 0x1e) << 3)
						  | ((b & 1) << 12) | ((b & 0x1e) >> 1);

				video_clut16[color] = MAKECOLOR(r1, g1, b1);
			}
		}
	}

	memset(neogeo_vidram16, 0, sizeof(neogeo_vidram16));
	memset(neogeo_palettebank16, 0, sizeof(neogeo_palettebank16));
	memset(video_palettebank, 0, sizeof(video_palettebank));

	for (i = 0; i < 0x1000; i += 16)
	{
		video_palettebank[0][i] = 0x8000;
		video_palettebank[1][i] = 0x8000;
	}

	video_buffer = SDL_CreateRGBSurface(SDL_SWSURFACE, 352, 256, 16, 0xF800, 0x07E0, 0x001F, 0x0000);
	game_title = NULL;

	ptr = (unsigned short *)(video_buffer->pixels);

	for(y=0;y<256;y++) {
		video_full_ptr[y] = ptr;
		if((y > 15) && (y < 240))
			video_line_ptr[y - 16] = ptr;
		ptr += 352;
	}

	ptr = (unsigned short *)(screen->pixels) + 320*8;

	for(y=0;y<224;y++) {
		video_line_ptr[y] = ptr;
		ptr += 320;
	}

	src.x=24;
	src.y=16;
	src.w=304;
	src.h=224;

	dest.x=8;
	dest.y=8;

	src.x=16;
	src.y=16;
	src.w=320;
	src.h=224;

	dest.x=0;
	dest.y=8;

	snapshot_init();

	for (i=0;i<256;i++)
	{
		int dday = 0;
		for (y=0;y<32;y++)
		{
			int j;
			int yskip = 0;
			ddayskip[i][y][0] = 0;
			for(j=0;j<16;j++) {
				if (j != 15)
					ddayskip[i][y][j+1]=0;
				dday -= i+1;
				if(dday <= 0)
				{
					dday += 256;
					yskip++;
					ddayskip[i][y][yskip]++;
				} else
					ddayskip[i][y][yskip]++;
			}
			ddaysize[i][y] = yskip;
		}
	}

	neogeo_video_reset();

	return 1;
}

//----------------------------------------------------------------------------
void video_shutdown(void)
{
	if (video_buffer != NULL) SDL_FreeSurface( video_buffer );
	if (game_title != NULL)   SDL_FreeSurface( game_title );
	video_buffer = game_title = NULL;
}

//----------------------------------------------------------------------------
void neogeo_video_reset(void)
{
	int i;

	neogeo_paletteram16 = neogeo_palettebank16[0];
	video_palette = video_palettebank[0];
	neogeo_palette_index = 0;
	neogeo_vidram16_modulo = 1;
	neogeo_vidram16_offset = 0;

	//next_update_first_line = FIRST_VISIBLE_LINE;

	video_enable = 1;
	spr_disable = 0;
	fix_disable = 0;

	for(i=0;i<32768;i++)
		video_spr_usage[i]=1;
}

//----------------------------------------------------------------------------
void video_draw_title(void)
{
	const int tileatr = 0;
	int count = 0;
	int x, y;
    dda_x_skip=ddaxskip[15];

	for(y=0;y<80;y+=16)
	{
		for(x=0;x<144;x+=16)
		{
			dda_y_skip = ddayskip[255][y>>4];
            DRAWTILE(count, x, y, 16, 16, count, 0, 0,
                (unsigned char*)video_buffer->pixels);
			count++;
		}
	}

	if(game_title) SDL_FreeSurface(game_title);
    game_title = SDL_CreateRGBSurface(SDL_SWSURFACE, 144,  80, 16, 0xF800, 0x07E0, 0x001F, 0x0000);
	SDL_BlitSurface(video_buffer, NULL, game_title, NULL);
	SDL_FillRect(video_buffer, NULL, video_palette[4095]);
	SDL_FillRect(screen, NULL, video_palette[4095]);

	now_loading();
}

//----------------------------------------------------------------------------
#define DRAW_SPRITE()												\
	if (sy2 + zy >= beginline && sy2 <= endline)					\
	{																\
		tileno = sprite_base[0];									\
		tileatr = sprite_base[1];									\
																	\
		if (tileatr & 0x08)											\
			tileno = (tileno & ~7) | (neogeo_frame_counter & 7);	\
		else if (tileatr & 0x04)									\
			tileno = (tileno & ~3) | (neogeo_frame_counter & 3);	\
																	\
		tileno &= 0x7FFF;											\
																	\
		if (video_spr_usage[tileno]) {								\
			dda_y_skip = ddayskip[rzy][y];							\
			DRAWTILE(tileno, sx + 16, sy2, rzx, zy,					\
					tileatr>>8, tileatr & 0x01, tileatr & 0x02,		\
                    (unsigned char*)video_buffer->pixels);			\
		}															\
	}

void video_draw_spr(int start, int end, int scanline)
{
	int	sx = 0,sy = 0,my = 0,rzx=16,rzy = 0;
	int sy2,zx,zy,i,y;
	int	tileno,tileatr;
	int fullmode=0;
	int offset;
	int scale, yctrl;
	register int beginline = startline;
	register int endline = (scanline > 239) ? 239 : scanline;
	unsigned short *sprite_base;

	for (offset = start; offset < end; offset++)
	{
		scale = sprite_scale[offset];
		yctrl = sprite_yctrl[offset];

		// If this bit is set this new column is placed next to last one
		if (yctrl & 0x40) {
			sx += rzx;
		} else {   // nope it is a new block
			rzy = scale & 0xff;
			sy = 0x200 - (yctrl >> 7);
			my = yctrl & 0x3f;

			if (my > 0x20)
			{
				fullmode = 1;
				my = 0x20;

				if (sy > 0x100) sy -= 0x200;
				while (sy < -16) sy += (rzy + 1) << 1;
			}
			else
				fullmode = 0;

			sx = sprite_xctrl[offset] >> 7;
		}

		if (sx >= 0x1f0) sx -= 0x200;
		if (sx < -16 || sx > 319) continue;
		if (!my || !rzy) continue;

		zx = (scale >> 8) & 0x0f;
        dda_x_skip = ddaxskip[zx];
        rzx = zx+1;
		sprite_base = &neogeo_vidram16[offset << 6];

		if (fullmode)
		{
			sy2 = sy;
			for (y = 0; y < my; y++)
			{
				if ((zy = ddaysize[rzy][y]))
				{
					if (sy2 >= 248) sy2 -= (rzy + 1) << 1;
					DRAW_SPRITE()
					sy2 += zy;
				}

				sprite_base += 2;
			}
		}
		else
		{
			int drawn_lines = 0;
			y = 0;
			while (drawn_lines < my << 4)
			{
				if ((zy = ddaysize[rzy][y]))
				{
					sy2 = (sy + drawn_lines) & 0x1ff;
					DRAW_SPRITE()
					drawn_lines += zy;
				}

				y++;
				if (y == 0x10 || y == 0x20)
					drawn_lines += (0xff - rzy) << 1;

				sprite_base += 2;
			}
		}
	}  // for offset
}

static void patch_vram_rbff2(void)
{
	int offs, tileno, tileatr;

	for (offs = 0; offs < ((0x300 >> 1) << 6) ; offs += 2)
	{
		unsigned short tileno  = neogeo_vidram16[offs];
		unsigned short tileatr = neogeo_vidram16[offs + 1];

		if (tileno == 0x7a00 && (tileatr == 0x4b00 || tileatr == 0x1400))
		{
			neogeo_vidram16[offs] = 0x7ae9;
			return;
		}
	}
}

static void patch_vram_adkworld(void)
{
	int offs, tileno, tileatr;

	for (offs = 0; offs < ((0x300 >> 1) << 6) ; offs += 2)
	{
		tileno  = neogeo_vidram16[offs];
		tileatr = neogeo_vidram16[offs + 1];

		if ((tileno == 0x14c0 || tileno == 0x1520) && tileatr == 0x0000)
			neogeo_vidram16[offs] = 0x0000;
	}
}

static void patch_vram_crsword2(void)
{
	int offs, tileno, tileatr;

	for (offs = 0; offs < ((0x300 >> 1) << 6) ; offs += 2)
	{
		tileno  = neogeo_vidram16[offs];
		tileatr = neogeo_vidram16[offs + 1];

		if (tileno == 0x52a0 && tileatr == 0x0000)
			neogeo_vidram16[offs] = 0x0000;
	}
}


void video_draw_screen(int scanline)
{
	static unsigned int	fc = 0;

	if (!skip_next_frame) {
		if (video_enable)
		{
			if ((!spr_disable) && (startline < 240))
			{
				if (neogeo_ngh == NGH_ssrpg)
				{
					int start = 0;
		
					if ((sprite_yctrl[4 >> 1] & 0x40) == 0 &&
						(sprite_yctrl[6 >> 1] & 0x40) != 0)
					{
						start = 6 >> 1;
		
						while ((sprite_yctrl[start] & 0x40) != 0)
							start++;
		
						if (start == 6 >> 1) start = 0;
					}
		
					video_draw_spr(start, 0x300 >> 1, scanline);
					if(start != 0)
						video_draw_spr(0, start, scanline);
				} else {
					switch (neogeo_ngh)
					{
						case NGH_crsword2: patch_vram_crsword2(); break;
						case NGH_adkworld: patch_vram_adkworld(); break;
						case NGH_rbff2: patch_vram_rbff2(); break;
					}
		
					video_draw_spr(0, 0x300 >> 1, scanline);
				}
			}
		}
		blitter(scanline);
	}

	if (fc >= neogeo_frame_counter_speed) {
		neogeo_frame_counter++;
		fc=3;
	}

	fc++;
}

#ifdef ARMASM
static __inline__ void draw_tile_gp2x(unsigned int tileno,int sx,int sy,int zx,int zy,int color,int flip)
{
	if (flip & 0x02) sy += zy - 1;
	DrawTileARM[zx][flip](tileno, color, video_full_ptr[sy]+sx, zy);
}
#elif !defined(X86ASM)
static __inline__ void draw_tile(unsigned int tileno,int sx,int sy,int zx,int zy,
					 int color,int xflip,int yflip,unsigned char *bmp)
{
    unsigned int *gfxdata,myword;
    unsigned char col;
    unsigned short *br;
    unsigned int *paldata = &video_palette[16*color];
    int y;
    int l = 0;
    int buf_w=(video_buffer->pitch>>1)-zx;
    int buf_w_yflip=(video_buffer->pitch>>1)+zx;
    gfxdata = (unsigned int *)&neogeo_spr_memory[tileno<<7];

    if (xflip) {
        l=0;
        if (yflip) {
            br= (unsigned short *)bmp+((zy-1)+sy)*(video_buffer->pitch>>1)+sx;
            for(y=0;y<zy;y++) {
                gfxdata+=dda_y_skip[l]<<1;
                myword = gfxdata[1];
                if (dda_x_skip[ 0]) {if ((col=((myword>>28)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 1]) {if ((col=((myword>>24)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 2]) {if ((col=((myword>>20)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 3]) {if ((col=((myword>>16)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 4]) {if ((col=((myword>>12)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 5]) {if ((col=((myword>> 8)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 6]) {if ((col=((myword>> 4)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 7]) {if ((col=((myword>> 0)&0xf))) *br = paldata[col];br++;}

                myword = gfxdata[0];
                if (dda_x_skip[ 8]) {if ((col=((myword>>28)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 9]) {if ((col=((myword>>24)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[10]) {if ((col=((myword>>20)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[11]) {if ((col=((myword>>16)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[12]) {if ((col=((myword>>12)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[13]) {if ((col=((myword>> 8)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[14]) {if ((col=((myword>> 4)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[15]) {if ((col=((myword>> 0)&0xf))) *br = paldata[col];br++;}
                l++;
                br-=buf_w_yflip;
            }
        } else {
            br= (unsigned short *)bmp+(sy)*(video_buffer->pitch>>1)+sx;
            for(y=0;y<zy;y++) {
                gfxdata+=dda_y_skip[l]<<1;
                myword = gfxdata[1];
                if (dda_x_skip[ 0]) {if ((col=((myword>>28)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 1]) {if ((col=((myword>>24)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 2]) {if ((col=((myword>>20)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 3]) {if ((col=((myword>>16)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 4]) {if ((col=((myword>>12)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 5]) {if ((col=((myword>> 8)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 6]) {if ((col=((myword>> 4)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 7]) {if ((col=((myword>> 0)&0xf))) *br = paldata[col];br++;}

                myword = gfxdata[0];
                if (dda_x_skip[ 8]) {if ((col=((myword>>28)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 9]) {if ((col=((myword>>24)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[10]) {if ((col=((myword>>20)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[11]) {if ((col=((myword>>16)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[12]) {if ((col=((myword>>12)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[13]) {if ((col=((myword>> 8)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[14]) {if ((col=((myword>> 4)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[15]) {if ((col=((myword>> 0)&0xf))) *br = paldata[col];br++;}
                l++;
                br+=buf_w;
            }
        }
    }else {
        if (yflip) {
            br= (unsigned short *)bmp+((zy-1)+sy)*(video_buffer->pitch>>1)+sx;
            for(y=0;y<zy;y++) {
                gfxdata+=dda_y_skip[l]<<1;
                myword = gfxdata[0];
                if (dda_x_skip[ 0]) {if ((col=((myword>> 0)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 1]) {if ((col=((myword>> 4)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 2]) {if ((col=((myword>> 8)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 3]) {if ((col=((myword>>12)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 4]) {if ((col=((myword>>16)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 5]) {if ((col=((myword>>20)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 6]) {if ((col=((myword>>24)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 7]) {if ((col=((myword>>28)&0xf))) *br = paldata[col];br++;}

                myword = gfxdata[1];
                if (dda_x_skip[ 8]) {if ((col=((myword>> 0)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 9]) {if ((col=((myword>> 4)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[10]) {if ((col=((myword>> 8)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[11]) {if ((col=((myword>>12)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[12]) {if ((col=((myword>>16)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[13]) {if ((col=((myword>>20)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[14]) {if ((col=((myword>>24)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[15]) {if ((col=((myword>>28)&0xf))) *br = paldata[col];br++;}
                br-=buf_w_yflip;
                l++;
            }
        } else {
            br= (unsigned short *)bmp+(sy)*(video_buffer->pitch>>1)+sx;
            for(y=0;y<zy;y++) {
                gfxdata+=dda_y_skip[l]<<1;
                myword = gfxdata[0];
                if (dda_x_skip[ 0]) {if ((col=((myword>> 0)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 1]) {if ((col=((myword>> 4)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 2]) {if ((col=((myword>> 8)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 3]) {if ((col=((myword>>12)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 4]) {if ((col=((myword>>16)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 5]) {if ((col=((myword>>20)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 6]) {if ((col=((myword>>24)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 7]) {if ((col=((myword>>28)&0xf))) *br = paldata[col];br++;}

                myword = gfxdata[1];
                if (dda_x_skip[ 8]) {if ((col=((myword>> 0)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[ 9]) {if ((col=((myword>> 4)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[10]) {if ((col=((myword>> 8)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[11]) {if ((col=((myword>>12)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[12]) {if ((col=((myword>>16)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[13]) {if ((col=((myword>>20)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[14]) {if ((col=((myword>>24)&0xf))) *br = paldata[col];br++;}
                if (dda_x_skip[15]) {if ((col=((myword>>28)&0xf))) *br = paldata[col];br++;}

                br+=buf_w;
                l++;
            }
        }
    }
}
#endif

/*------------------------------------------------------
	Refresh loading screen(progress)
------------------------------------------------------*/
#define BTN(x) (1 << x)
int neogeo_loading_screenrefresh(int flag, int draw)
{
	static TICKER prev;
	static int limit;
	static u32 prevbtn = 0;
	u32 buttons = read_joystick(0);
	u32 change = prevbtn ^ buttons;
	prevbtn = buttons;

	if (flag)
	{
		prev = get_ticks() - 16667; //TICKS_PER_SEC / FPS;
		limit = neogeo_cdspeed_limit;
	}

	if (limit)
	{
		TICKER target = prev + 16667; //TICKS_PER_SEC / FPS;
		TICKER curr = get_ticks();

		while (curr < target) curr = get_ticks();

		prev = curr;
	}

	if (change & buttons & BTN(P1_SELECT))
	{
		limit = !limit;
	}

	if (limit || draw)
	{
		SDL_FillRect(video_buffer,NULL,video_palette[4095]);
		if (video_enable)
		{
			if (!spr_disable)
				video_draw_spr(0, 0x300 >> 1, RASTER_LINES);
			if (!fix_disable)
				video_draw_fix();
		}
		blitter(RASTER_LINES);
	}

	return 0;
}

void snapshot_init(void)
{
	/* This function sets up snapshot number */
	char name[30];
	FILE *fp;

	snap_no=0;

	sprintf(name,"snap%04d.bmp", snap_no);

	while((fp=fopen(name,"r"))!=NULL)
	{
		fclose(fp);
		snap_no++;
		sprintf(name,"snap%04d.bmp", snap_no);
	}
}

void video_save_snapshot(void)
{
	char name[30];
	sprintf(name,"snap%04d.bmp", snap_no++);
	SDL_SaveBMP(screen,name);
}


//------------------------------------------------------------------------
// Blitters
//------------------------------------------------------------------------

void blitter(int scanline)
{
	if(startline > scanline) return;

	if(startline <= LAST_VISIBLE_LINE) {
		src.y = startline;
		src.h = ((scanline > LAST_VISIBLE_LINE) ? 240 : scanline + 1) - startline;
		startline = scanline + 1;
		dest.y = src.y - 8;
		SDL_BlitSurface(video_buffer,&src,screen,&dest);
	}

	if(scanline == RASTER_LINES)
	{

		if (!fix_disable) //&& (scanline == RASTER_LINES))
			video_draw_fix();
		SDL_Flip(screen);
		SDL_FillRect(video_buffer,NULL,video_palette[4095]);
		startline = 16;
	}
}
