#include "neocd.h"
#include "z80intrf.h"
#include "DrZ80.h"

static struct DrZ80 drz80;

static unsigned int z80_rebasePC(unsigned short address)
{
	drz80.Z80PC_BASE = (unsigned int)memory_region_cpu2;
	drz80.Z80PC = drz80.Z80PC_BASE + address;
	return drz80.Z80PC_BASE + address;
}

static unsigned int z80_rebaseSP(unsigned short address)
{
	drz80.Z80SP_BASE = (unsigned int)memory_region_cpu2;
	drz80.Z80SP = drz80.Z80SP_BASE + address;
	return drz80.Z80SP_BASE + address;
}

static unsigned char z80_read8(unsigned short address) {
	return memory_region_cpu2[address];
}

static unsigned short z80_read16(unsigned short address) {
	return memory_region_cpu2[address] | (memory_region_cpu2[address + 1] << 8);
}

static void z80_write8(unsigned char data,unsigned short address) {
	memory_region_cpu2[address] = data;
}

static void z80_write16(unsigned short data,unsigned short address) {
	memory_region_cpu2[address] = data & 0xFF;
	memory_region_cpu2[address + 1] = data >> 8;
}

/****************************************************************************
 * Reset registers to their initial values
 ****************************************************************************/
void z80_init(void)
{
	memset (&drz80, 0, sizeof(drz80));

  	drz80.z80_rebasePC=z80_rebasePC;
  	drz80.z80_rebaseSP=z80_rebaseSP;
  	drz80.z80_read8   =z80_read8;
  	drz80.z80_read16  =z80_read16;
  	drz80.z80_write8  =z80_write8;
  	drz80.z80_write16 =z80_write16;
  	drz80.z80_in      =neogeo_z80_port_r;
  	drz80.z80_out     =neogeo_z80_port_w;

	drz80.Z80A = 0x00 <<24;
	drz80.Z80F = (1<<2); /* set ZFlag */
	drz80.Z80BC = 0x0000 <<16;
	drz80.Z80DE = 0x0000 <<16;
	drz80.Z80HL = 0x0000 <<16;
	drz80.Z80A2 = 0x00 <<24;
	drz80.Z80F2 = 1<<2;  /* set ZFlag */
	drz80.Z80BC2 = 0x0000 <<16;
	drz80.Z80DE2 = 0x0000 <<16;
	drz80.Z80HL2 = 0x0000 <<16;
	drz80.Z80IX = 0xFFFF;// <<16;
	drz80.Z80IY = 0xFFFF;// <<16;
	drz80.Z80I = 0x00;
	drz80.Z80IM = 0x01;
	drz80.Z80_IRQ = 0x00;
	drz80.Z80IF = 0x00;
	drz80.Z80PC=drz80.z80_rebasePC(0);
	drz80.Z80SP=drz80.z80_rebaseSP(0xffff);/*0xf000;*/
}

void z80_exec(int cycles)
{
	DrZ80Run(&drz80, cycles);
}

void z80_nmi(void)
{
	//printf("Cause NMI %x %x\n",mydrz80.Z80_IRQ,mydrz80.Z80IF);
	drz80.Z80_IRQ |= 0x02;
	//DrZ80Run(&mydrz80, 30);
}
void z80_raise_irq(int l)
{
	//printf("raise irq %x %x\n",mydrz80.Z80_IRQ,mydrz80.Z80IF);
	drz80.Z80_IRQ |= 0x1;
	//mydrz80.z80irqvector= l & 0xff;
}
void z80_lower_irq(void)
{
	//printf("Lower irq %x %x\n",mydrz80.Z80_IRQ,mydrz80.Z80IF);
	drz80.Z80_IRQ &= ~0x1;
}
void z80_reset(void)
{
	z80_init();
}
